part of calendar;

//// Action performed in data source
enum CalendarDataSourceAction { Add, Remove, Reset }

/// Defines the data source of [SfCalendar] widget.
/// Calendar data source used to store the appointment related properties.
///
/// ```dart
///Widget build(BuildContext context) {
///   return Container(
///      child: SfCalendar(
///        view: CalendarView.day,
///        dataSource: _getCalendarDataSource(),
///      ),
///    );
///  }
///
/// class MeetingDataSource extends CalendarDataSource {
///  MeetingDataSource(this.source);
///  List<_Meeting> source;
///
///  @override
///  List<dynamic> get dataSource => source;
///
///  @override
///  AppointmentMapper<DateTime> get startTimeMapper => (int index) => source[index].from;
///
///  @override
///  AppointmentMapper<DateTime> get endTimeMapper => (int index) => source[index].to;
///
///  @override
///  AppointmentMapper<bool> get isAllDayMapper => (int index) => source[index].isAllDay;
///
///  @override
///  AppointmentMapper<String> get subjectMapper => (int index) => source[index].title;
///
///  @override
///  AppointmentMapper<String> get startTimeZoneMapper => (int index) => source[index].fromZone;
///
///  @override
///  AppointmentMapper<String> get endTimeZoneMapper => (int index) => source[index].toZone;
///
///  @override
///  AppointmentMapper<String> get recurrenceRuleMapper => (int index) => source[index].recurrenceRule;
///
///  @override
///  AppointmentMapper<List<DateTime>> get recurrenceExceptionDatesMapper => (int index) => source[index].exceptionDates;
///
///  @override
///  AppointmentMapper<Color> get colorMapper => (int index) => source[index].background;
/// }
///
/// class Meeting {
///  Meeting(
///      {this.from,
///      this.to,
///      this.title,
///      this.isAllDay,
///      this.background,
///      this.fromZone,
///      this.toZone,
///      this.exceptionDates,
///      this.recurrenceRule});
///
///  DateTime from;
///  DateTime to;
///  String title;
///  bool isAllDay;
///  Color background;
///  String fromZone;
///  String toZone;
///  String recurrenceRule;
///  List<DateTime> exceptionDates;
/// }
///
/// final DateTime date = DateTime.now();
///  MeetingDataSource _getCalendarDataSource() {
///    List<Meeting> appointments = <Meeting>[];
///    appointments.add(Meeting(
///     from: date,
///     to: date.add(const Duration(hours: 1)),
///     title: 'General Meeting',
///     isAllDay: false,
///     background: Colors.red,
///     fromZone: '',
///     toZone: '',
///     recurrenceRule: '',
///     exceptionDates: null
///  ));
///
///    return MeetingDataSource(appointments);
///  }
///  ```
abstract class CalendarDataSource extends CalendarDataSourceChangeNotifier {
  //// Defines the events/appointments to the calendar.
  List<dynamic> dataSource;

  //// Start time mapper used to map the custom appointment start time property
  //// with [Appointment] start time property.
  AppointmentMapper<DateTime> startTimeMapper;

  //// End time mapper used to map the custom appointment end time property
  //// with [Appointment] end time property.
  AppointmentMapper<DateTime> endTimeMapper;

  //// Subject mapper used to map the custom appointment subject property
  //// with [Appointment] subject property.
  AppointmentMapper<String> subjectMapper;

  //// Is all day mapper used to map the custom appointment is all day property
  //// with [Appointment] is all day property.
  AppointmentMapper<bool> isAllDayMapper;

  //// Color mapper used to map the custom appointment color property
  //// with [Appointment] color property.
  AppointmentMapper<Color> colorMapper;

  //// Notes mapper used to map the custom appointment notes property
  //// with [Appointment] notes property.
  AppointmentMapper<String> notesMapper;

  //// Location mapper used to map the custom appointment location property
  //// with [Appointment] location property.
  AppointmentMapper<String> locationMapper;

  //// Start time zone mapper used to map the custom appointment start time zone property
  //// with [Appointment] start time zone property.
  AppointmentMapper<String> startTimeZoneMapper;

  //// end time zone mapper used to map the custom appointment end time zone property
  //// with [Appointment] end time zone property.
  AppointmentMapper<String> endTimeZoneMapper;

  //// Recurrence rule mapper used to map the custom appointment recurrence rule property
  //// with [Appointment] recurrence rule property.
  AppointmentMapper<String> recurrenceRuleMapper;

  //// Recurrence exception dates mapper used to map the custom appointment recurrence exception dates property
  //// with [Appointment] recurrence exception dates property.
  AppointmentMapper<List<DateTime>> recurrenceExceptionDatesMapper;
}

//// Data source call back used to perform operation while data source changed.
typedef CalendarDataSourceCallback = void Function(
    CalendarDataSourceAction, List<dynamic>);

//// Data source notifier used to notify the action performed in data source
class CalendarDataSourceChangeNotifier {
  List<CalendarDataSourceCallback> _listeners;

  //// Add the listener used to call back when data source changed.
  void addListener(CalendarDataSourceCallback _listener) {
    _listeners ??= <CalendarDataSourceCallback>[];
    _listeners.add(_listener);
  }

  //// remove the listener used for notify the data source changes.
  void removeListener(CalendarDataSourceCallback _listener) {
    if (_listeners == null) {
      return;
    }

    _listeners.remove(_listener);
  }

  //// Notify to the listener while data source changed.
  void notifyListeners(CalendarDataSourceAction type, List<dynamic> data) {
    if (_listeners == null) {
      return;
    }

    for (CalendarDataSourceCallback listener in _listeners) {
      if (listener != null) {
        listener(type, data);
      }
    }
  }

  @mustCallSuper
  void dispose() {
    _listeners = null;
  }
}
